// Copyright © 2009, Esko Luontola <www.orfjackal.net>
// This software is released under the Apache License 2.0.
// The license text is at http://www.apache.org/licenses/LICENSE-2.0

package addressbook.gui;

import addressbook.domain.Contact;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.table.AbstractTableModel;
import java.awt.*;
import java.util.*;
import java.util.List;

/**
 * @author Esko Luontola
 */
public class AddressBookViewImpl implements AddressBookView {

    private final JPanel root = new JPanel();
    private final JTextField searchField = new JTextField();
    private final JTable contactsTable;
    private final ContactsTableModel contactsModel;

    private final AddressBookController controller;

    public AddressBookViewImpl(AddressBookController controller) {
        this.controller = controller;
        contactsModel = new ContactsTableModel();
        contactsTable = new JTable(contactsModel);

        initEventListeners();
        initLayout();
    }

    private void initEventListeners() {
        searchField.getDocument().addDocumentListener(new DocumentChangeListener() {
            public void onDocumentChanged(DocumentEvent e) {
                controller.fireSearchKeywordsEntered(searchField.getText());
            }
        });
    }

    private void initLayout() {
        searchField.setColumns(15);

        JPanel toolbar = new JPanel();
        toolbar.setLayout(new FlowLayout(FlowLayout.RIGHT));
        toolbar.add(new JLabel("Search"));
        toolbar.add(searchField);

        root.setLayout(new BorderLayout());
        root.add(toolbar, BorderLayout.NORTH);
        root.add(new JScrollPane(contactsTable), BorderLayout.CENTER);
    }

    public JComponent getComponent() {
        return root;
    }

    public void updateContacts(List<Contact> contacts) {
        contactsModel.updateContacts(contacts);
    }


    private class ContactsTableModel extends AbstractTableModel {

        private final String[] COLUMN_NAMES = new String[]{
                "Name", "Address", "Phone", "Email"
        };

        private final List<Contact> contacts = new ArrayList<Contact>();

        public void updateContacts(List<Contact> newContacts) {
            contacts.clear();
            contacts.addAll(newContacts);
            fireTableDataChanged();
        }

        public String getColumnName(int column) {
            return COLUMN_NAMES[column];
        }

        public int getRowCount() {
            return contacts.size();
        }

        public int getColumnCount() {
            return COLUMN_NAMES.length;
        }

        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return true;
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            Contact contact = contacts.get(rowIndex);
            String[] columns = new String[]{
                    contact.getName(),
                    contact.getAddress(),
                    contact.getPhone(),
                    contact.getEmail()
            };
            return columns[columnIndex];
        }

        public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
            String newValue = (String) aValue;
            Contact contact = contacts.get(rowIndex);
            switch (columnIndex) {
                case 0:
                    controller.fireNameChanged(contact, newValue);
                    break;
                case 1:
                    controller.fireAddressChanged(contact, newValue);
                    break;
                case 2:
                    controller.firePhoneChanged(contact, newValue);
                    break;
                case 3:
                    controller.fireEmailChanged(contact, newValue);
                    break;
            }
        }
    }
}
