// Copyright © 2009, Esko Luontola <www.orfjackal.net>
// This software is released under the Apache License 2.0.
// The license text is at http://www.apache.org/licenses/LICENSE-2.0

package addressbook.domain;

import static addressbook.domain.Dummies.*;
import net.orfjackal.nestedjunit.NestedJUnit4;
import org.junit.*;
import org.junit.runner.RunWith;

import java.util.*;

/**
 * @author Esko Luontola
 */
@RunWith(NestedJUnit4.class)
public class AddressBookTest extends Assert {

    private final AddressBook addressBook = new AddressBook();

    public class An_empty_addresbook {

        @Test
        public void has_no_contacts() {
            assertEquals(0, addressBook.size());
        }

        @Test
        public void has_contacts_after_one_is_added() {
            addressBook.add(TOM);
            assertEquals(1, addressBook.size());
            assertEquals(TOM, addressBook.get(0));
        }
    }

    public class An_address_book_with_many_contacts {

        @Before
        public void addManyContacts() {
            addressBook.add(TOM);
            addressBook.add(DICK);
            addressBook.add(HARRY);
        }

        @Test
        public void has_all_those_contacts() {
            assertEquals(3, addressBook.size());
        }

        @Test
        public void the_contacts_are_sorted_by_name() {
            assertEquals(DICK, addressBook.get(0));
            assertEquals(HARRY, addressBook.get(1));
            assertEquals(TOM, addressBook.get(2));
        }

        @Test
        public void the_contacts_can_be_searched_by_name() {
            List<Contact> found = addressBook.search(TOM.getName());
            assertContainsInOrder(found, TOM);
        }

        @Test
        public void the_contacts_can_be_searched_by_address() {
            List<Contact> found = addressBook.search(TOM.getAddress());
            assertContainsInOrder(found, TOM);
        }

        @Test
        public void the_contacts_can_be_searched_by_phone() {
            List<Contact> found = addressBook.search(DICK.getPhone());
            assertContainsInOrder(found, DICK);
        }

        @Test
        public void the_contacts_can_be_searched_by_email() {
            List<Contact> found = addressBook.search(DICK.getEmail());
            assertContainsInOrder(found, DICK);
        }

        @Test
        public void the_search_is_case_insensitive() {
            List<Contact> found = addressBook.search(HARRY.getName().toUpperCase());
            assertContainsInOrder(found, HARRY);
        }

        @Test
        public void the_search_is_substring_search() {
            List<Contact> found = addressBook.search("Street");
            assertContainsInOrder(found, DICK, HARRY, TOM);
        }
    }

    public class When_a_contact_in_the_address_book_is_updated {

        private Contact updatedHarry;

        @Before
        public void addContactsAndUpdateOne() {
            addressBook.add(TOM);
            addressBook.add(DICK);
            addressBook.add(HARRY);

            updatedHarry = HARRY.withAddress("Broadway");
            addressBook.update(HARRY, updatedHarry);
        }

        @Test
        public void the_new_info_is_added_to_the_addressbook() {
            List<Contact> found = addressBook.search(updatedHarry.getAddress());
            assertContainsInOrder(found, updatedHarry);
        }

        @Test
        public void the_new_info_is_removed_from_the_addressbook() {
            List<Contact> found = addressBook.search(HARRY.getAddress());
            assertContainsInOrder(found);
        }

        @Test
        public void the_other_contacts_are_not_affected() {
            List<Contact> all = addressBook.search("");
            assertTrue(all.contains(TOM));
            assertTrue(all.contains(DICK));
        }
    }


    private static <T> void assertContainsInOrder(List<T> actual, T... expected) {
        String errorMessage = String.format("\nexpected %s\n but was %s\n", Arrays.toString(expected), actual);
        assertEquals(errorMessage + "Wrong number of elements:", expected.length, actual.size());
        for (int i = 0; i < expected.length; i++) {
            assertEquals(errorMessage + "Wrong element at index " + i + ":", expected[i], actual.get(i));
        }
    }
}
