// Copyright © 2009, Esko Luontola <www.orfjackal.net>
// This software is released under the Apache License 2.0.
// The license text is at http://www.apache.org/licenses/LICENSE-2.0

package addressbook.gui;

import addressbook.domain.*;
import static addressbook.domain.Dummies.*;
import net.orfjackal.nestedjunit.NestedJUnit4;
import org.junit.*;
import org.junit.runner.RunWith;

import java.util.List;

/**
 * @author Esko Luontola
 */
@RunWith(NestedJUnit4.class)
public class AddressBookControllerTest extends Assert {

    private AddressBook addressBook;
    private AddressBookController controller;
    private FakeAddressBookView view;

    @Before
    public void createTestData() {
        addressBook = new AddressBook();
        addressBook.add(TOM);
        addressBook.add(DICK);
        addressBook.add(HARRY);

        view = new FakeAddressBookView();
        controller = new AddressBookController(addressBook);
        controller.setView(view);
    }


    public class When_the_view_is_opened {

        @Test
        public void it_is_populated_with_all_contacts() {
            assertTrue(view.contacts.containsAll(addressBook.search("")));
        }

        @Test
        public void the_last_row_is_empty_to_allow_adding_new_contacts() {
            Contact lastRow = view.contacts.get(view.contacts.size() - 1);
            assertEquals("", lastRow.getName());
        }
    }

    public class When_search_keywords_are_entered {

        @Before
        public void enterSearchKeywords() {
            controller.fireSearchKeywordsEntered(TOM.getName());
        }

        @Test
        public void the_view_is_populated_with_the_search_results() {
            assertTrue(view.contacts.contains(TOM));
            assertFalse(view.contacts.contains(DICK));
        }
    }

    public class When_the_name_of_a_new_contact_is_entered {
        private int editedRow;

        @Before
        public void enterNameForNewContact() {
            editedRow = view.contacts.size() - 1;
            controller.fireNameChanged(view.contacts.get(editedRow), "Alphonse");
        }

        @Test
        public void it_is_added_to_the_address_book() {
            List<Contact> found = addressBook.search("Alphonse");
            assertEquals(1, found.size());
        }

        @Test
        public void it_is_added_to_the_edited_row_in_the_view() {
            Contact contact = view.contacts.get(editedRow);
            assertEquals("Alphonse", contact.getName());
        }

        @Test
        public void a_new_empty_row_is_added_to_the_bottom_of_the_view() {
            int lastRow = view.contacts.size() - 1;
            assertEquals(editedRow + 1, lastRow);
            assertEquals("", view.contacts.get(lastRow).getName());
        }
    }

    public class When_an_existing_contact_is_edited {
        private final String NEW_NAME = "Wilhelm";
        private final String NEW_ADDRESS = "Hollywood";
        private final String NEW_PHONE = "9090";
        private final String NEW_EMAIL = "wilhelm@hollywood.com";

        private int editedRow;
        private Contact editedContact;

        @Before
        public void init() {
            editedRow = 0;
            editedContact = view.contacts.get(editedRow);
        }

        @Test
        public void its_name_is_changed_in_the_view() {
            controller.fireNameChanged(editedContact, NEW_NAME);

            assertEquals(NEW_NAME, view.contacts.get(editedRow).getName());
        }

        @Test
        public void its_name_is_changed_in_the_address_book() {
            controller.fireNameChanged(editedContact, NEW_NAME);

            assertEquals(1, addressBook.search(NEW_NAME).size());
        }

        @Test
        public void its_address_is_changed_likewise() {
            controller.fireAddressChanged(editedContact, NEW_ADDRESS);

            assertEquals(NEW_ADDRESS, view.contacts.get(editedRow).getAddress());
            assertEquals(1, addressBook.search(NEW_ADDRESS).size());
        }

        @Test
        public void its_phone_is_changed_likewise() {
            controller.firePhoneChanged(editedContact, NEW_PHONE);

            assertEquals(NEW_PHONE, view.contacts.get(editedRow).getPhone());
            assertEquals(1, addressBook.search(NEW_PHONE).size());
        }

        @Test
        public void its_email_is_changed_likewise() {
            controller.fireEmailChanged(editedContact, NEW_EMAIL);

            assertEquals(NEW_EMAIL, view.contacts.get(editedRow).getEmail());
            assertEquals(1, addressBook.search(NEW_EMAIL).size());
        }
    }
}
